;******************************************************************************
; HEADER
;******************************************************************************

	TITLE		"Universal Remote Switch Box"
	SUBTITLE	"Copyright 2008, Jeroen R. H. Hoppenbrouwers"
	LIST		P=PIC16F877
	INCLUDE		P16F877.INC
	ERRORLEVEL	-302
	ERRORLEVEL	-202
	__CONFIG	_XT_OSC & _WDT_ON & _PWRTE_ON & _BODEN_ON & _LVP_OFF & _CPD_OFF & _WRT_ENABLE_ON & _CP_OFF
	; OSCILLATOR	XT 4.000 MHz

;******************************************************************************
; RAM ALLOCATION
;******************************************************************************

LARGE_COUNT	EQU 0x20			; TIMER LARGE COUNT
SMALL_COUNT	EQU 0x21			; TIMER SMALL COUNT
LEDS		EQU 0x22			; LEDS
FLAGS		EQU 0x23			; FLAGS
ADRESS_BUFFER	EQU 0x24			; ADRESS BUFFER
TEMP_VALUE	EQU 0x25			; TEMP VALUE

UIR_OUTPUT1	EQU 0x26			; OUTPUT BYTE 1
UIR_OUTPUT2	EQU 0x27			; OUTPUT BYTE 2
UIR_OUTPUT3	EQU 0x28			; OUTPUT BYTE 3
UIR_OUTPUT4	EQU 0x29			; OUTPUT BYTE 4
UIR_OUTPUT5	EQU 0x2A			; OUTPUT BYTE 5
UIR_OUTPUT6	EQU 0x2B			; OUTPUT BYTE 6
UIR_BUFFER3	EQU 0x2C			; UIR BUFFER
UIR_BUFFER2	EQU 0x2D			; UIR BUFFER
UIR_BUFFER1	EQU 0x2E			; UIR BUFFER

PROG_RXBUFFER_1	EQU 0x2F			; PROG RX BUFFER 1
PROG_RXBUFFER_2	EQU 0x30			; PROG RX BUFFER 2
PROG_RXBUFFER_3	EQU 0x31			; PROG RX BUFFER 3
PROG_RXBUFFER_4	EQU 0x32			; PROG RX BUFFER 4
PROG_RXBUFFER_5	EQU 0x33			; PROG RX BUFFER 5
PROG_RXBUFFER_6	EQU 0x34			; PROG RX BUFFER 6

TX_DATA		EQU 0x35			; TX DATA
RX_DATA		EQU 0x36			; RX DATA
BIT_COUNTER	EQU 0x37			; BIT COUNTER
CHECKSUM	EQU 0x38			; CHECKSUM

W_TEMP		EQU 0x70			; STORAGE W REGISTER
STATUS_TEMP	EQU 0x71			; STORAGE STATUS
PCLATH_TEMP	EQU 0x72			; STORAGE PCLATH
EEPROM_COUNTER	EQU 0x73			; EEPROM BYTE COUNTER
EEPROM_ADDRESS	EQU 0x74			; EEPROM ADDRESS BUFFER
EEPROM_TEMP	EQU 0x75			; EEPROM TEMP VALUE

TOGGLE1		EQU 0x76			; TOGGLE/PULSE SETTING FOR PORTB
TOGGLE2		EQU 0x77			; TOGGLE/PULSE SETTING FOR PORTD
MASTER1		EQU 0x78			; MASTER/SLAVE SETTING FOR PORTB
MASTER2		EQU 0x79			; MASTER/SLAVE SETTING FOR PORTD
TOGGLE_TEMP1	EQU 0x7A			; TOGGLE/PULSE TEMP SETTING FOR PORTB
TOGGLE_TEMP2	EQU 0x7B			; TOGGLE/PULSE TEMP SETTING FOR PORTD

RAM_BUFFER1	EQU 0x110			; RAM BUFFER 1 IN BANK 3
RAM_BUFFER2	EQU 0x130			; RAM BUFFER 2 IN BANK 3
RAM_BUFFER3	EQU 0x150			; RAM BUFFER 3 IN BANK 3
RAM_BUFFER4	EQU 0x190			; RAM BUFFER 4 IN BANK 4
RAM_BUFFER5	EQU 0x1B0			; RAM BUFFER 5 IN BANK 4
RAM_BUFFER6	EQU 0x1D0			; RAM BUFFER 6 IN BANK 4

;******************************************************************************
; FLAGS
;******************************************************************************

PROG_MODE_1	EQU 0x00			; PROGRAMMING MODE FLAG
PROG_MODE_2	EQU 0x01			; PROGRAMMING MODE FLAG (2ND RECEIVE)
TOGGLE_HOLD	EQU 0x02			; TOGGLE HOLD FLAG
RX_FLAG		EQU 0x03			; RX FLAG
STORE_EEPROM	EQU 0x04			; STORE EEPROM FLAG
WDT_TIMEOUT	EQU 0x05			; WDT TIME OUT

;******************************************************************************
; DEFINES
;******************************************************************************

#DEFINE		RXD		PORTC,7		; RXD INPUT
#DEFINE		TXD		PORTC,6		; TXD OUTPUT
#DEFINE		ENABLE		PORTC,5		; DRIVER ENABLE
#DEFINE		IR_INPUT	PORTA,5		; IR INPUT

#DEFINE		BUTTON1		PORTC,0		; BUTTON 1
#DEFINE		BUTTON2		PORTC,1		; BUTTON 2
#DEFINE		BUTTON3		PORTC,2		; BUTTON 3

;******************************************************************************
; MAIN PROGRAM
;******************************************************************************

		ORG	0x00
START		GOTO	INIT			; POWER UP INIT
		ORG	0x04
IRQ		GOTO	IRQ_HANDLER		; IRQ

;******************************************************************************

LED_CONV	ANDLW	0x0F			; MASK OUT HIGH NIBBLE
		ADDWF	PCL,F			; CALCULATE OFFSET

		RETLW	0x24			; 1 GREEN
		RETLW	0x21			; 1 RED
		RETLW	0x14			; 2 GREEN
		RETLW	0x12			; 2 RED
		RETLW	0x42			; 3 GREEN
		RETLW	0x41			; 3 RED

		RETLW	0x01			; 1 RED & 3 RED
		RETLW	0x02			; 2 RED & 3 GREEN
		RETLW	0x03			; 1 RED & 2 RED
		RETLW	0x04			; 1 GREEN & 2 GREEN
		RETLW	0x05			; 2 GREEN & 3 RED
		RETLW	0x06			; 1 GREEN & 3 GREEN

		RETLW	0x00			; LED OFF
		RETLW	0x00			; LED OFF
		RETLW	0x00			; LED OFF
		RETLW	0x00			; LED OFF

PORT_CONV	ANDLW	0x0F			; MASK OUT HIGH NIBBLE
		ADDWF	PCL,F			; CALCULATE OFFSET

		RETLW	0x80			; OUTPUT 0
		RETLW	0x40			; OUTPUT 1
		RETLW	0x20			; OUTPUT 2
		RETLW	0x10			; OUTPUT 3
		RETLW	0x08			; OUTPUT 4
		RETLW	0x04			; OUTPUT 5
		RETLW	0x02			; OUTPUT 6
		RETLW	0x01			; OUTPUT 7

		RETLW	0x80			; OUTPUT 8
		RETLW	0x40			; OUTPUT 9
		RETLW	0x20			; OUTPUT A
		RETLW	0x10			; OUTPUT B
		RETLW	0x01			; OUTPUT C
		RETLW	0x08			; OUTPUT D
		RETLW	0x02			; OUTPUT E
		RETLW	0x04			; OUTPUT F

;******************************************************************************
; IRQ HANDLER
;******************************************************************************

IRQ_HANDLER	MOVWF	W_TEMP			; COPY W TO TEMP REGISTER
		SWAPF	STATUS,W		; SWAP STATUS TO W
		CLRF 	STATUS			; BANK 0, REGARDLESS OF CURRENT BANK - CLEARS IRP,RP1,RP0
		MOVWF	STATUS_TEMP		; SAVE STATUS TO BANK 0
		MOVF 	PCLATH, W		; ONLY REQUIRED IF USING PAGES 1, 2 AND/OR 3
		MOVWF	PCLATH_TEMP		; SAVE PCLATH INTO W
		CLRF 	PCLATH			; PAGE ZERO, REGARDLESS OF CURRENT PAGE
		CLRWDT				; RESET WATCHDOG
		BTFSC	PIR1,TMR1IF		; TIMER1 IRQ ?
		GOTO	TIMER1_IRQ		; YES

END_IRQ		MOVF 	PCLATH_TEMP, W		; RESTORE PCLATH
		MOVWF	PCLATH			; MOVE W INTO PCLATH
		SWAPF	STATUS_TEMP,W		; SETS BANK TO ORIGINAL STATE
		MOVWF	STATUS			; MOVE W INTO STATUS REGISTER
		SWAPF	W_TEMP,F		; SWAP W_TEMP
		SWAPF	W_TEMP,W		; SWAP W_TEMP INTO W
		RETFIE				; RETURN FROM IRQ

;******************************************************************************

TIMER1_IRQ	BCF	PIR1,TMR1IF		; CLEAR TIMER FLAG
		COMF	MASTER1,W		; LOAD & INVERT SETTING
		IORWF	TOGGLE1,W		; OR WITH TOGGLE SETTING
		ANDWF	PORTB,F			; AND WITH OUTPUTS (CLEAR PULSE OUTPUTS)
		COMF	MASTER2,W		; LOAD & INVERT SETTING
		IORWF	TOGGLE2,W		; OR WITH TOGGLE SETTING
		ANDWF	PORTD,F			; AND WITH OUTPUTS (CLEAR PULSE OUTPUTS)
		BCF	FLAGS,TOGGLE_HOLD	; CLEAR HOLD FLAG
		GOTO	END_IRQ			; END IRQ

;******************************************************************************

INIT		BCF	STATUS,RP0		; SELECT BANK 0
		BCF	STATUS,RP1		; SELECT BANK 0
		BCF	FLAGS,WDT_TIMEOUT	; CLEAR WDT TIME OUT FLAG
		BTFSS	STATUS,NOT_TO		; TO SET?
		BSF	FLAGS,WDT_TIMEOUT	; NO, WDT TIME OUT
		
		CLRWDT				; CLEAR WDT
		CLRF	INTCON			; NO IRQ
		BCF	PIR1,TMR1IF		; CLEAR TIMER FLAG
		BSF	STATUS,RP0		; SELECT BANK 1
		BSF	PIE1,TMR1IE		; SET TIMER IRQ
		MOVLW	0x06			; LOAD 0x06
		MOVWF	ADCON1			; ALL DIGITAL
		MOVLW	0x3F			; LOAD 0x3F
		MOVWF	TRISA			; SET PORTA
		MOVLW	0x00			; LOAD 0x00
		MOVWF	TRISB			; SET PORTB
		MOVLW	0x9F			; LOAD 0xFF
		MOVWF	TRISC			; SET PORTC
		MOVLW	0x00			; LOAD 0x00
		MOVWF	TRISD			; SET PORTD
		MOVLW	0x00			; LOAD 0x00
		MOVWF	TRISE			; SET PORTE
		MOVLW	0xC2			; LOAD 0x07
		MOVWF	OPTION_REG		; TIMER 0 PRE 1:8
		BCF	STATUS,RP0		; SELECT BANK 0
		CLRF	ADCON0			; DISABLE ADC
		
		BTFSC	FLAGS,WDT_TIMEOUT	; WDT TIMEOUT?
		GOTO	INIT_SKIP		; YES
		CLRF	PORTA			; CLEAR PORTA
		CLRF	PORTB			; CLEAR PORTB
		CLRF	PORTC			; CLEAR PORTC
		CLRF	PORTD			; CLEAR PORTD
		CLRF	PORTE			; CLEAR PORTE
		CLRF	FLAGS			; CLEAR FLAGS

INIT_SKIP	BSF	ENABLE			; SET OUTPUT
		CLRF	LEDS			; CLEAR LEDS
		MOVLW	0x21			; LOAD 0x21
		MOVWF	T1CON			; TIMER 1 ON; PRESCALER 1:4
		CLRF	TMR1L			; CLEAR TIMER 1
		CLRF	TMR1H			; CLEAR TIMER 1
		BSF	INTCON,PEIE		; SET PERIPHERAL IRQ
		BSF	INTCON,GIE		; SET GLOBAL IRQ
		CALL	READ_CONFIG		; READ EEPROM

		BTFSC	FLAGS,WDT_TIMEOUT	; WDT TIMEOUT?
		GOTO	UIR_WDT			; YES

MAIN		BTFSC	FLAGS,PROG_MODE_1	; PROG MODE ?
		GOTO	PROG_MODE		; YES, PROG MODE
		MOVLW	0x0B			; LED 1 & 3 GREEN
		CALL	LED_SET			; SET LED
		CALL	UIR_INIT		; INIT TIMER
LOOP		CLRWDT				; CLEAR WDT
		BTFSS	BUTTON1			; BUTTON 1 PRESSED?
		GOTO	PROG_MODE_KEY		; YES
		BTFSS	BUTTON2			; BUTTON 2 PRESSED?
		GOTO	MASTER_KEY		; YES
		BTFSS	BUTTON3			; BUTTON 3 PRESSED?
		GOTO	EDIT_MODE		; YES
		BTFSC	IR_INPUT		; READ INPUT
		GOTO	LOOP			; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
		GOTO	UIR			; UIR

;******************************************************************************

LED_SET		CALL	LED_CONV		; CALL TABLE
		MOVWF	LEDS			; STORE
		ANDLW	0x07			; LOWER BITS
		MOVWF	PORTE			; TO PORT
		SWAPF	LEDS,W			; SWAP IN W
		ANDLW	0x07			; LOWER BITS
		BSF	STATUS,RP0		; BANK 1
		MOVWF	TRISE			; TO TRIS
		BCF	STATUS,RP0		; BANK 0
		RETURN				; RETURN

;******************************************************************************

PROG_MODE_KEY	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON1			; BUTTON 1 STILL PRESSED?
		GOTO	MAIN			; NO, BACK TO MAIN
PROG_MODE	MOVLW	0x01			; LED 1 RED
		CALL	LED_SET			; SET LED
		CALL	UIR_INIT		; INIT TIMER
PROG_MODE_RX	CLRWDT				; CLEAR WDT
		BTFSS	BUTTON2			; EXIT BUTTON PRESSED?
		GOTO	PROG_EXIT		; YES, EXIT
		BSF	FLAGS,PROG_MODE_1	; SET FLAG
		BTFSC	IR_INPUT		; READ INPUT
		GOTO	PROG_MODE_RX		; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
		GOTO	UIR			; UIR

PROG_EXIT	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON2			; EXIT BUTTON STILL PRESSED?
		GOTO	PROG_MODE		; NO, BACK TO PROG
		CALL	WRITE_CONFIG		; STORE IN EEPROM
		BCF	FLAGS,PROG_MODE_1	; CLEAR FLAG
		BCF	FLAGS,PROG_MODE_2	; CLEAR FLAG
		CALL	READ_CONFIG		; READ EEPROM
		GOTO	MAIN			; BACK TO MAIN

;******************************************************************************

MASTER_KEY	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON2			; BUTTON 2 STILL PRESSED?
		GOTO	MAIN			; NO, BACK TO MAIN
		CLRWDT				; CLEAR WDT
		MOVLW	0xEE			; LOAD 0xEE
		MOVWF	UIR_OUTPUT1		; STORE IN BUFFER 1
		MOVWF	UIR_OUTPUT2		; STORE IN 2
		MOVWF	UIR_OUTPUT3		; STORE IN 3
		MOVWF	UIR_OUTPUT4		; STORE IN 4
		MOVWF	UIR_OUTPUT5		; STORE IN 5
		MOVWF	UIR_OUTPUT6		; STORE IN 6
		GOTO	RECEIVED_CMD		; EXECUTE CMD

;******************************************************************************

EDIT_MODE	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON3			; BUTTON 3 STILL PRESSED?
		GOTO	MAIN			; NO, BACK TO MAIN
EDIT_MODE_0	BCF	ENABLE			; SET INPUT
		CALL	READ_CONFIG		; READ EEPROM
EDIT_MODE_1	MOVLW	0x05			; NO, LED 3 RED
		CALL	LED_SET			; SET LED
		CLRWDT				; CLEAR WDT
		BTFSS	BUTTON1			; DELETE BUTTON 1 PRESSED?
		GOTO	EDIT_DELETE		; YES, DELETE
		BTFSS	BUTTON2			; EXIT BUTTON PRESSED?
		GOTO	EDIT_EXIT		; YES, EXIT
		BTFSC	RXD			; START BIT ?
		GOTO	EDIT_MODE_1		; START BIT NOT FOUND

		CALL	READ_FIRSTBYTE		; READ FIRST BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVLW	0xAA			; LOAD SYNC VALUE
		SUBWF	RX_DATA,W		; SUBTRACT RECEIVED BYTE
		BTFSC	STATUS,Z		; UPLOAD CMD?
		GOTO	UPLOAD			; YES UPLOAD
		ANDLW	0xFE			; MASK OUT LOW NIBBLE
		BTFSC	STATUS,Z		; DOWNLOAD CMD?
		GOTO	DOWNLOAD		; YES DOWNLOAD
		GOTO	EDIT_MODE_1		; LOOP

EDIT_DELETE	BTFSC	BUTTON3			; DELETE BUTTON 2 PRESSED?
		GOTO	EDIT_MODE_1		; NO, LOOP
		MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON1			; DELETE BUTTON 1 STILL PRESSED?
		GOTO	EDIT_MODE_1		; NO, LOOP
		BTFSC	BUTTON3			; DELETE BUTTON 2 STILL PRESSED?
		GOTO	EDIT_MODE_1		; NO, LOOP

		COMF	PORTA,W			; LOAD & INVERT
		ANDLW	0x1F			; 5 BITS
		MOVWF	ADRESS_BUFFER		; STORE

		BSF	STATUS,IRP		; BANK 2 & 3
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER1		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CLRF	INDF			; DELETE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER2		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CLRF	INDF			; DELETE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER3		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CLRF	INDF			; DELETE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER4		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CLRF	INDF			; DELETE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER5		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CLRF	INDF			; DELETE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER6		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CLRF	INDF			; DELETE
		BCF	STATUS,IRP		; BANK 0 & 1
		BSF	FLAGS,STORE_EEPROM	; SET STORE FLAG
		MOVLW	0x03			; LED 2 RED
		CALL	LED_SET			; SET LED
		MOVLW	0xFA			; LOAD 250
		CALL	WAIT_MS			; WAIT 250MS
		GOTO	EDIT_MODE_1		; LOOP

EDIT_EXIT	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON2			; EXIT BUTTON STILL PRESSED?
		GOTO	EDIT_MODE_1		; NO, LOOP
		BTFSC	FLAGS,STORE_EEPROM	; STORE FLAG SET?
		CALL	WRITE_CONFIG		; YES, STORE IN EEPROM
		BCF	FLAGS,STORE_EEPROM	; CLEAR STORE FLAG
		BSF	ENABLE			; SET OUTPUT
		CALL	READ_CONFIG		; READ EEPROM
		GOTO	MAIN			; BACK TO MAIN

;******************************************************************************

DOWNLOAD	BSF	ENABLE			; SET OUTPUT
		MOVLW	0x14			; LOAD 20
		CALL	WAIT_MS			; WAIT 20MS
		MOVLW	0xAA			; LOAD STARTBYTE
		CALL	SEND_BYTE		; SEND DATA
		CLRF	ADRESS_BUFFER		; CLEAR ADRESS BUFFER
		CLRF	CHECKSUM		; CLEAR CHECKSUM
DOWNLOAD_NEXT	CLRWDT				; CLEAR WDT
		BSF	STATUS,IRP		; BANK 2 & 3
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER1		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD DATA
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER2		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD DATA
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER3		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD DATA
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER4		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD DATA
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER5		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD DATA
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER6		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD DATA
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA

		INCF	ADRESS_BUFFER,F		; INC POINTER
		MOVLW	0x20			; 32 BLOKS (192 BYTES)
		XORWF	ADRESS_BUFFER,W		; COMPARE POINTER
		BTFSS	STATUS,Z		; READY?
		GOTO	DOWNLOAD_NEXT		; NO, NEXT BYTE

		MOVF	TOGGLE1,W		; LOAD TOGGLE 1
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	MASTER1,W		; LOAD MASTER 1
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	TOGGLE2,W		; LOAD TOGGLE 2
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVF	MASTER2,W		; LOAD MASTER 2
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	SEND_BYTE		; SEND DATA

		MOVF	CHECKSUM,W		; LOAD CHECKSUM
		CALL	SEND_BYTE		; SEND DATA
		MOVLW	0x14			; LOAD 20
		CALL	WAIT_MS			; WAIT 20MS
		BCF	ENABLE			; SET INPUT
		GOTO	EDIT_MODE_0		; RETURN & READ EEPROM

;******************************************************************************

UPLOAD		CLRF	ADRESS_BUFFER		; CLEAR POINTER
		CLRF	CHECKSUM		; CLEAR CHECKSUM

UPLOAD_NEXT	CLRWDT				; CLEAR WDT
		BSF	STATUS,IRP		; BANK 2 & 3
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER1		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	INDF			; STORE RAM
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER2		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	INDF			; STORE RAM
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER3		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	INDF			; STORE RAM
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER4		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	INDF			; STORE RAM
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER5		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	INDF			; STORE RAM
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER6		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	INDF			; STORE RAM
		XORWF	CHECKSUM,F		; XOR CHECKSUM

		INCF	ADRESS_BUFFER,F		; INC POINTER
		MOVLW	0x20			; 32 BLOKS (192 BYTES)
		XORWF	ADRESS_BUFFER,W		; COMPARE POINTER
		BTFSS	STATUS,Z		; READY?
		GOTO	UPLOAD_NEXT		; NO, NEXT BYTE

		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	TOGGLE1			; STORE TOGGLE 1
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	MASTER1			; STORE MASTER 1
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	TOGGLE2			; STORE TOGGLE 2
		XORWF	CHECKSUM,F		; XOR CHECKSUM
		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE_1		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		MOVWF	MASTER2			; STORE MASTER 2
		XORWF	CHECKSUM,F		; XOR CHECKSUM

		CALL	READ_BYTE		; READ BYTE
		BTFSS	FLAGS,RX_FLAG		; TEST RECEIVE FLAG
		GOTO	EDIT_MODE		; RETURN
		MOVF	RX_DATA,W		; LOAD DATA
		XORWF	CHECKSUM,W		; COMPARE DATA
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	SEND_NAK		; SEND NAK

		BSF	ENABLE			; SET OUTPUT
		MOVLW	0x14			; LOAD 20
		CALL	WAIT_MS			; WAIT 20MS
		MOVLW	0x06			; LOAD ACK
		CALL	SEND_BYTE		; SEND DATA
		BCF	ENABLE			; SET INPUT
		CALL	WRITE_CONFIG		; STORE IN EEPROM
		GOTO	EDIT_MODE_0		; RETURN & READ EEPROM

SEND_NAK	BSF	ENABLE			; SET OUTPUT
		MOVLW	0x14			; LOAD 20
		CALL	WAIT_MS			; WAIT 20MS
		MOVLW	0x15			; LOAD NAK
		CALL	SEND_BYTE		; SEND DATA
		MOVLW	0x14			; LOAD 20
		CALL	WAIT_MS			; WAIT 20MS
		BCF	ENABLE			; SET INPUT
		GOTO	EDIT_MODE_0		; RETURN & READ EEPROM

;******************************************************************************

READ_BYTE	BTFSC	RXD			; START BIT = 0 ?
		GOTO	READ_BYTE		; NO, WAIT
READ_FIRSTBYTE	CLRF	RX_DATA			; CLR DATA REGISTER
		BCF	FLAGS,RX_FLAG		; CLEAR RECEIVE FLAG
		CALL	WAIT_BITHALF		; WAIT 1/2 BIT TIME
		BTFSC	RXD			; START BIT = 0 ?
		RETURN				; NO, EXIT
		MOVLW	0x08			; 8 DATA BITS
		MOVWF	BIT_COUNTER		; TO BITCOUUNTER
READ_NEXTBIT	CALL	WAIT_BITTIME		; WAIT 1 BIT TIME
		BCF	STATUS,C		; CLR CARRY 
		RRF	RX_DATA,F		; LSB FIRST (SHIFT RIGHT)
		BTFSC	RXD			; IS THE BIT 0 OR 1 ?
		BSF	RX_DATA,7		; BIT = 1  (SET BIT 7) 
		DECFSZ	BIT_COUNTER,F		; DEC BITCOUNTER SKIP IF CLEAR
		GOTO	READ_NEXTBIT		; NOT READY GET NEXT BIT
		CALL	WAIT_BITTIME		; WAIT 1 BIT TIME
		BTFSS	RXD			; STOP BIT
		RETURN				; RETRURN
		BSF	FLAGS,RX_FLAG		; SET RECEIVE FLAG
		RETURN				; RETRURN

;******************************************************************************

SEND_BYTE	MOVWF	TX_DATA			; STORE DATA
		MOVLW	0x08			; LOAD 8
		MOVWF	BIT_COUNTER		; INTO BITCOUNTER
		BCF	TXD			; SEND STARTBIT
		CALL	WAIT_BITTIME		; WAIT BITTIME
SEND_NEXTBIT	BCF	STATUS,C		; CLR CARRY
		RRF	TX_DATA,F		; SHIFT BUFFER
		BTFSS	STATUS,C		; TEST CARRY
		GOTO	SEND_ZERO		; BIT = 0
		BSF	TXD			; SEND BIT = 1
		GOTO	SEND_ONE		; CONTINUE
SEND_ZERO	BCF	TXD			; SEND BIT = 0
SEND_ONE	CALL	WAIT_BITTIME		; WAIT BITTIME
		DECFSZ	BIT_COUNTER,F		; 8 BITS SEND?
		GOTO	SEND_NEXTBIT		; NEXT BIT
		BSF	TXD			; SEND STOPBIT
		CALL	WAIT_BITTIME		; WAIT BITTIME
		RETURN				; RETURN

;******************************************************************************

RECEIVED_CMD	CLRWDT				; CLEAR WDT
		MOVF	UIR_OUTPUT1,W		; LOAD BUFFER 1
		IORWF	UIR_OUTPUT2,W		; OR BUFFER 2
		IORWF	UIR_OUTPUT3,W		; OR BUFFER 3
		IORWF	UIR_OUTPUT4,W		; OR BUFFER 4
		IORWF	UIR_OUTPUT5,W		; OR BUFFER 5
		IORWF	UIR_OUTPUT6,W		; OR BUFFER 6
		BTFSC	STATUS,Z		; 0 ?
		GOTO	MAIN			; EXIT

		BTFSC	FLAGS,PROG_MODE_2	; PROG MODE 2 ?
		GOTO	PROG_RX2		; YES
		BTFSC	FLAGS,PROG_MODE_1	; PROG MODE 1 ?
		GOTO	PROG_RX1		; YES

		CLRF	ADRESS_BUFFER		; CLEAR ADRESS BUFFER
RECEIVED_LOOP	CLRWDT				; CLEAR WDT
		BSF	STATUS,IRP		; BANK 2 & 3

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER1		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD BUFFER
		XORWF	UIR_OUTPUT1,W		; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	NEXT_BLOCK		; NO

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER2		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD BUFFER
		XORWF	UIR_OUTPUT2,W		; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	NEXT_BLOCK		; NO

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER3		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD BUFFER
		XORWF	UIR_OUTPUT3,W		; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	NEXT_BLOCK		; NO

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER4		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD BUFFER
		XORWF	UIR_OUTPUT4,W		; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	NEXT_BLOCK		; NO

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER5		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD BUFFER
		XORWF	UIR_OUTPUT5,W		; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	NEXT_BLOCK		; NO

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER6		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	INDF,W			; LOAD BUFFER
		XORWF	UIR_OUTPUT6,W		; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	NEXT_BLOCK		; NO

		CLRF	TMR1L			; CLEAR TIMER 1
		CLRF	TMR1H			; CLEAR TIMER 1
		BCF	PIR1,TMR1IF		; CLEAR TIMER FLAG

		BTFSC	FLAGS,TOGGLE_HOLD	; HOLD FLAG SET?
		GOTO	MAIN_D			; YES, SKIP
		BSF	FLAGS,TOGGLE_HOLD	; SET HOLD FLAG

		MOVF	ADRESS_BUFFER,W		; LOAD ADRESS BUFFER
		CALL	PORT_CONV		; CONVERT PORT
		MOVWF	TEMP_VALUE		; STORE TEMP VALUE
		BTFSC	ADRESS_BUFFER,3		; OUTPUT 0...7?
		GOTO	NEXT_OUTPUTS		; NO, OUTPUT 8...F

		MOVF	TOGGLE1,W		; LOAD SETTING
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSC	STATUS,Z		; 0 ?
		GOTO	PULSE_MODE_1		; PULSE OR SLAVE MODE
		MOVF	MASTER1,W		; LOAD SETTING
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	MASTER_MODE_1		; MASTER MODE
TOGGLE_MODE_1	MOVF	TEMP_VALUE,W		; LOAD TEMP VALUE
		XORWF	PORTB,F			; XOR PORTB (TOGGLE MODE)
		GOTO	MAIN_D			; READY

MASTER_MODE_1	MOVF	PORTB,W			; LOAD OUTPUTS
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSC	STATUS,Z		; 0 ?
		GOTO	TOGGLE_MODE_1		; TOGGLE OUTPUT ON
		MOVF	TOGGLE1,W		; LOAD SETTING
		IORWF	MASTER1,W		; OR SETTING
		ANDWF	PORTB,F			; AND PORTB (SLAVES OFF)
		MOVF	TOGGLE2,W		; LOAD SETTING
		IORWF	MASTER2,W		; OR SETTING
		ANDWF	PORTD,F			; AND PORTD (SLAVES OFF)
		GOTO	TOGGLE_MODE_1		; TOGGLE OUTPUT OFF

PULSE_MODE_1	MOVF	TEMP_VALUE,W		; LOAD TEMP VALUE
		IORWF	PORTB,F			; OR PORTB (PULSE MODE)
		MOVF	MASTER1,W		; LOAD SETTING
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	MAIN_D			; READY
SLAVE_MODE_1	MOVF	TOGGLE1,W		; LOAD SETTING
		ANDWF	MASTER1,W		; AND SETTING
		IORWF	PORTB,F			; OR PORTB (MASTERS ON)
		MOVF	TOGGLE2,W		; LOAD SETTING
		ANDWF	MASTER2,W		; AND SETTING
		IORWF	PORTD,F			; OR PORTD (MASTERS ON)
		GOTO	MAIN_D			; READY

NEXT_OUTPUTS	MOVF	TOGGLE2,W		; LOAD SETTING
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSC	STATUS,Z		; 0 ?
		GOTO	PULSE_MODE_2		; PULSE OR SLAVE MODE
		MOVF	MASTER2,W		; LOAD SETTING
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	MASTER_MODE_2		; MASTER MODE
TOGGLE_MODE_2	MOVF	TEMP_VALUE,W		; LOAD TEMP VALUE
		XORWF	PORTD,F			; XOR PORTD (TOGGLE MODE)
		GOTO	MAIN_D			; READY

MASTER_MODE_2	MOVF	PORTD,W			; LOAD OUTPUTS
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSC	STATUS,Z		; 0 ?
		GOTO	TOGGLE_MODE_2		; TOGGLE OUTPUT ON
		MOVF	TOGGLE1,W		; LOAD SETTING
		IORWF	MASTER1,W		; OR SETTING
		ANDWF	PORTB,F			; AND PORTB (SLAVES OFF)
		MOVF	TOGGLE2,W		; LOAD SETTING
		IORWF	MASTER2,W		; OR SETTING
		ANDWF	PORTD,F			; AND PORTD (SLAVES OFF)
		GOTO	TOGGLE_MODE_2		; TOGGLE OUTPUT OFF

PULSE_MODE_2	MOVF	TEMP_VALUE,W		; LOAD TEMP VALUE
		IORWF	PORTD,F			; OR PORTB (PULSE MODE)
		MOVF	MASTER2,W		; LOAD SETTING
		ANDWF	TEMP_VALUE,W		; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	MAIN_D			; READY
SLAVE_MODE_2	MOVF	TOGGLE1,W		; LOAD SETTING
		ANDWF	MASTER1,W		; AND SETTING
		IORWF	PORTB,F			; OR PORTB (MASTERS ON)
		MOVF	TOGGLE2,W		; LOAD SETTING
		ANDWF	MASTER2,W		; AND SETTING
		IORWF	PORTD,F			; OR PORTD (MASTERS ON)
		GOTO	MAIN_D			; READY

MAIN_D		CLRWDT				; CLEAR WDT
		MOVLW	0x03			; LED 2 RED
		CALL	LED_SET			; SET LED
		CALL	WAIT_1MS		; WAIT 1 MS
		MOVLW	0xFF			; LOAD STARTBYTE
		CALL	SEND_BYTE		; SEND DATA
		MOVF	ADRESS_BUFFER,W		; LOAD CMD
		IORLW	0x40			; CONVERT TO ASCII
		CALL	SEND_BYTE		; SEND DATA
		MOVLW	0xFE			; LOAD STARTBYTE
		CALL	SEND_BYTE		; SEND DATA
		CALL	WAIT_1MS		; WAIT 1 MS
		GOTO	MAIN			; RETURN HOME

NEXT_BLOCK	INCF	ADRESS_BUFFER,F		; INC ADRESS BUFFER
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		XORLW	0x20			; 32 BLOCKS
		BTFSS	STATUS,Z		; WRAP?
		GOTO	RECEIVED_LOOP		; NO, NEXT BLOCK
		MOVLW	0x0F			; LEDS OFF
		CALL	LED_SET			; SET LED
		MOVLW	0x05			; LOAD 5
		CALL	WAIT_MS			; WAIT 5 MS
		BCF	STATUS,IRP		; BANK 0 & 1
		GOTO	MAIN			; RETURN HOME

;******************************************************************************

PROG_RX1	MOVF	UIR_OUTPUT1,W		; LOAD BUFFER
		MOVWF	PROG_RXBUFFER_1		; STORE BUFFER
		MOVF	UIR_OUTPUT2,W		; LOAD BUFFER
		MOVWF	PROG_RXBUFFER_2		; STORE BUFFER
		MOVF	UIR_OUTPUT3,W		; LOAD BUFFER
		MOVWF	PROG_RXBUFFER_3		; STORE BUFFER
		MOVF	UIR_OUTPUT4,W		; LOAD BUFFER
		MOVWF	PROG_RXBUFFER_4		; STORE BUFFER
		MOVF	UIR_OUTPUT5,W		; LOAD BUFFER
		MOVWF	PROG_RXBUFFER_5		; STORE BUFFER
		MOVF	UIR_OUTPUT6,W		; LOAD BUFFER
		MOVWF	PROG_RXBUFFER_6		; STORE BUFFER
		MOVLW	0x02			; LED 2 GREEN
		CALL	LED_SET			; SET LED
		MOVLW	0x05			; LOAD 5
		CALL	WAIT_MS			; WAIT 5 MS
		BSF	FLAGS,PROG_MODE_2	; SET FLAG
		GOTO	PROG_MODE		; BACK TO PROG MODE

PROG_RX2	MOVF	UIR_OUTPUT1,W		; LOAD BUFFER
		XORWF	PROG_RXBUFFER_1,W	; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	PROG_RX2A		; NO
		MOVF	UIR_OUTPUT2,W		; LOAD BUFFER
		XORWF	PROG_RXBUFFER_2,W	; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	PROG_RX2A		; NO
		MOVF	UIR_OUTPUT3,W		; LOAD BUFFER
		XORWF	PROG_RXBUFFER_3,W	; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	PROG_RX2A		; NO
		MOVF	UIR_OUTPUT4,W		; LOAD BUFFER
		XORWF	PROG_RXBUFFER_4,W	; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	PROG_RX2A		; NO
		MOVF	UIR_OUTPUT5,W		; LOAD BUFFER
		XORWF	PROG_RXBUFFER_5,W	; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	PROG_RX2A		; NO
		MOVF	UIR_OUTPUT6,W		; LOAD BUFFER
		XORWF	PROG_RXBUFFER_6,W	; COMPARE BUFFER
		BTFSS	STATUS,Z		; EQUAL?
		GOTO	PROG_RX2A		; NO
		MOVLW	0x09			; LED 1 & 2 GREEN
		CALL	LED_SET			; SET LED
		MOVLW	0x05			; LOAD 5
		CALL	WAIT_MS			; WAIT 5 MS
		GOTO	PROG_RX2B		; NEXT BLOCK

PROG_RX2A	MOVLW	0x0A			; LED 2 GREEN & LED 3 RED
		CALL	LED_SET			; SET LED
		MOVLW	0x0			; LOAD 5
		CALL	WAIT_MS			; WAIT 5 MS
		BCF	FLAGS,PROG_MODE_2	; SET FLAG
		GOTO	PROG_MODE		; BACK TO PROG MODE

PROG_RX2B	CLRWDT				; CLEAR WATCHDOG
		BCF	FLAGS,PROG_MODE_2	; SET FLAG
		BTFSS	BUTTON1			; RESTART BUTTON PRESSED?
		GOTO	PROG_RESTART		; YES, RESTART PROG MODE
		BTFSS	BUTTON2			; EXIT BUTTON PRESSED?
		GOTO	PROG_EXIT		; YES, EXIT
		BTFSS	BUTTON3			; STORE BUTTON PRESSED?
		GOTO	PROG_STORE		; YES, STORE DATA
		GOTO	PROG_RX2B		; LOOP

PROG_RESTART	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON1			; BUTTON 1 STILL PRESSED?
		GOTO	PROG_RX2B		; NO, GO BACK
		GOTO	PROG_MODE		; BACK TO PROG MODE

PROG_STORE	MOVLW	0x32			; LOAD 50
		CALL	WAIT_MS			; WAIT 50MS
		BTFSC	BUTTON3			; BUTTON 3 STILL PRESSED?
		GOTO	PROG_RX2B		; NO, GO BACK

		COMF	PORTA,W			; LOAD & INVERT
		ANDLW	0x1F			; 5 BITS
		MOVWF	ADRESS_BUFFER		; STORE

		BSF	STATUS,IRP		; BANK 2 & 3
		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER1		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	UIR_OUTPUT1,W		; LOAD BUFFER
		MOVWF	INDF			; STORE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER2		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	UIR_OUTPUT2,W		; LOAD BUFFER
		MOVWF	INDF			; STORE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER3		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	UIR_OUTPUT3,W		; LOAD BUFFER
		MOVWF	INDF			; STORE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER4		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	UIR_OUTPUT4,W		; LOAD BUFFER
		MOVWF	INDF			; STORE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER5		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	UIR_OUTPUT5,W		; LOAD BUFFER
		MOVWF	INDF			; STORE

		MOVF	ADRESS_BUFFER,W		; LOAD BUFFER
		ADDLW	RAM_BUFFER6		; ADD OFFSET
		MOVWF	FSR			; SET POINTER
		MOVF	UIR_OUTPUT6,W		; LOAD BUFFER
		MOVWF	INDF			; STORE
		BCF	STATUS,IRP		; BANK 0 & 1
		GOTO	PROG_MODE		; BACK TO PROG MODE

;******************************************************************************

UIR		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR03			; SKIP BIT
		MOVF	TMR0,W			; LOAD TIMER
		MOVWF	UIR_BUFFER2		; STORE IN RAM
		GOTO	UIR14			; GOTO PART 2
UIR03		CLRWDT				; CLEAR WDT
		BTFSS	TMR0,7			; TEST BIT 7
		GOTO	UIR			; LOOP 
		BTFSS	TMR0,6			; TEST BIT 6
		GOTO	UIR			; LOOP
		CLRWDT				; CLEAR WDT
UIR04		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR04			; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
		CLRWDT				; CLEAR WDT
UIR05		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR06			; SKIP BIT
		MOVF	TMR0,W			; LOAD TIMER
		CLRF	TMR0			; CLEAR TIMER
		MOVWF	UIR_BUFFER2		; STORE IN RAM
		GOTO	UIR26			; GOT PART 4
UIR06		BTFSS	TMR0,7			; TEST BIT 7
		GOTO	UIR05			; LOOP
		CLRWDT				; CLEAR WDT
UIR07		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR07			; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
UIR08		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR08			; WAIT FOR INPUT
		MOVF	TMR0,W			; LOAD TIMER
		CLRF	TMR0			; CLEAR TIMER
		MOVWF	UIR_BUFFER2		; STORE IN RAM
		BCF	STATUS,C		; CLEAR CARRY
		RRF	UIR_BUFFER2,W		; SHIFT TO W
		ADDWF	UIR_BUFFER2,F		; ADD BACK TO RAM
UIR09		CLRWDT				; CLEAR WDT
		CLRF	UIR_OUTPUT1		; CLEAR BUFFER
		CLRF	UIR_OUTPUT2		; CLEAR BUFFER
		CLRF	UIR_OUTPUT3		; CLEAR BUFFER
		CLRF	UIR_OUTPUT4		; CLEAR BUFFER
		CLRF	UIR_OUTPUT5		; CLEAR BUFFER
		CLRF	UIR_OUTPUT6		; CLEAR BUFFER
		MOVLW	0x30			; LOAD 0x30
		MOVWF	UIR_BUFFER1		; STORE IN RAM
		GOTO	UIR11			; SKIP BIT
UIR10		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR10			; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
		CLRWDT				; CLEAR WDT
UIR11		BCF	STATUS,C		; CLEAR CARRY
		MOVF	TMR0,W			; LOAD TIMER
		SUBWF	UIR_BUFFER2,W		; SUBSTRACT
		BTFSS	STATUS,Z		; ZERO ?
		GOTO	UIR11			; LOOP
		BCF	STATUS,C		; CLEAR CARRY
		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR13			; SKIP BIT
		CLRF	TMR0			; CLEAR TIMER
UIR12		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR12			; WAIT FOR INPUT
		BSF	STATUS,C		; SET CARRY
UIR13		CLRWDT				; CLEAR WDT
		RLF	UIR_OUTPUT6,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT5,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT4,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT3,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT2,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT1,F		; SHIFT BUFFER
		DECFSZ	UIR_BUFFER1,F		; DEC
		GOTO	UIR10			; LOOP
		GOTO	RECEIVED_CMD		; PROCESS COMMAND

;******************************************************************************

UIR14		CLRWDT				; CLEAR WDT
		BTFSC	UIR_BUFFER2,7		; TEST BIT 7
		GOTO	UIR20			; GOTO PART 3
		BTFSC	UIR_BUFFER2,6		; TEST BIT 6
		GOTO	UIR20			; GOTO PART 3
		CLRF	TMR0			; CLEAR TIMER
		CLRWDT				; CLEAR WDT
UIR15		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR15			; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
		CLRWDT				; CLEAR WDT
UIR16		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR16			; WAIT FOR INPUT
		MOVF	TMR0,W			; LOAD TIMER
		CLRF	TMR0			; CLEAR TIMER
		MOVWF	UIR_BUFFER2		; STORE IN RAM
		RRF	UIR_BUFFER2,F		; SHIFT BUFFER
		CLRWDT				; CLEAR WDT
UIR17		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR17			; WAIT FOR INPUT
		MOVF	TMR0,W			; LOAD TIMER
		CLRF	TMR0			; CLEAR TIMER
		ADDWF	UIR_BUFFER2,F		; ADD BACK TO RAM
		CLRWDT				; CLEAR WDT
UIR18		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR18			; WAIT FOR INPUT
		GOTO	UIR09			; LOOP
UIR19		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR19			; WAIT FOR INPUT
		MOVF	TMR0,W			; LOAD TIMER
		CLRF	TMR0			; CLEAR TIMER
		MOVWF	UIR_BUFFER2		; STORE IN RAM
		RRF	UIR_BUFFER2,F		; SHIFT BUFFER
		GOTO	UIR09			; LOOP

;******************************************************************************

UIR20		CLRWDT				; CLEAR WDT
		BCF	STATUS,C		; CLEAR CARRY
		RRF	UIR_BUFFER2,W		; SHIFT TO W
		ADDWF	UIR_BUFFER2,F		; ADD BACK TO RAM
		MOVLW	0x50			; LOAD 0x50
		MOVWF	UIR_BUFFER3		; STORE IN RAM
		CLRF	UIR_OUTPUT1		; CLEAR BUFFER
		CLRF	UIR_OUTPUT2		; CLEAR BUFFER
		CLRF	UIR_OUTPUT3		; CLEAR BUFFER
		CLRF	UIR_OUTPUT4		; CLEAR BUFFER
		CLRF	UIR_OUTPUT5		; CLEAR BUFFER
		CLRF	UIR_OUTPUT6		; CLEAR BUFFER
		MOVLW	0x31			; LOAD 0x31
		MOVWF	UIR_BUFFER1		; STORE IN RAM
UIR21		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR21			; WAIT FOR INPUT
		CLRWDT				; CLEAR WDT
		CLRF	TMR0			; CLEAR TIMER
UIR22		BCF	STATUS,C		; CLEAR CARRY
		MOVF	TMR0,W			; LOAD TIMER
		SUBWF	UIR_BUFFER2,W		; SUBSTRACT
		BTFSS	STATUS,Z		; ZERO?
		GOTO	UIR22			; LOOP
		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR24			; SKIP BIT
UIR23		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR23			; WAIT FOR INPUT
		BCF	STATUS,C		; CLEAR CARRY
		GOTO	UIR25			; SKIP BIT
UIR24		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR24			; WAIT FOR INPUT
		BSF	STATUS,C		; SET CARRY
UIR25		CLRF	TMR0			; CLEAR TIMER
		CLRWDT				; CLEAR WDT
		RLF	UIR_OUTPUT6,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT5,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT4,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT3,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT2,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT1,F		; SHIFT BUFFER
		DECFSZ	UIR_BUFFER1,F		; DEC BUFFER
		GOTO	UIR22			; LOOP
		GOTO	RECEIVED_CMD		; PROCESS COMMAND

;******************************************************************************

UIR26		CLRWDT				; CLEAR WDT
		BCF	STATUS,C		; CLEAR CARRY
		RRF	UIR_BUFFER2,F		; SHIFT BUFFER
		ADDWF	UIR_BUFFER2,W		; ADD TO W
		RRF	UIR_BUFFER2,F		; SHIFT BUFFER
		ADDWF	UIR_BUFFER2,F		; ADD BACK TO RAM
		MOVLW	0x53			; LOAD 0x53
		MOVWF	UIR_BUFFER3		; STORE IN RAM
		CLRF	UIR_OUTPUT1		; CLEAR BUFFER
		CLRF	UIR_OUTPUT2		; CLEAR BUFFER
		CLRF	UIR_OUTPUT3		; CLEAR BUFFER
		CLRF	UIR_OUTPUT4		; CLEAR BUFFER
		CLRF	UIR_OUTPUT5		; CLEAR BUFFER
		CLRF	UIR_OUTPUT6		; CLEAR BUFFER
		MOVLW	0x30			; LOAD 0x30
		MOVWF	UIR_BUFFER1		; STROE IN RAN
		GOTO	UIR28			; SKIP BIT
UIR27		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR27			; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
		CLRWDT				; CLEAR WDT
UIR28		BCF	STATUS,C		; CLEAR CARRY
		MOVF	TMR0,W			; LOAD TIMER
		SUBWF	UIR_BUFFER2,W		; SUBSTRACT
		BTFSS	STATUS,Z		; ZERO?
		GOTO	UIR28			; LOOP
		BCF	STATUS,C		; CLEAR CARRY
		BTFSC	IR_INPUT		; READ INPUT
		GOTO	UIR30			; SKIP BIT
		CLRF	TMR0			; CLEAR TIMER
UIR29		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR29			; WAIT FOR INPUT
		BSF	STATUS,C		; SET CARRY
UIR30		CLRWDT				; CLEAR WDT
		RLF	UIR_OUTPUT6,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT5,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT4,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT3,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT2,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT1,F		; SHIFT BUFFER
		DECFSZ	UIR_BUFFER1,F		; DEC BUFFER
		GOTO	UIR27			; LOOP
		GOTO	RECEIVED_CMD		; PROCESS COMMAND

;******************************************************************************

UIR_INIT	MOVLW	0xC7			; LOAR 0xC7
		MOVWF	OPTION_REG		; TIMER 0 PRE 1:256
		CLRWDT				; CLEAR WDT
		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR_INIT		; WAIT FOR INPUT
		CLRF	TMR0			; CLEAR TIMER
UIR_INIT2	CLRWDT				; CLEAR WDT
		BTFSS	IR_INPUT		; READ INPUT
		GOTO	UIR_INIT		; WAIT FOR INPUT
		BTFSS	TMR0,7			; TEST BIT 7
		GOTO	UIR_INIT2		; WAIT FOR TIMER
		MOVLW	0xC2			; LOAD 0xC2
		MOVWF	OPTION_REG		; TIMER 0 PRE 1:8
		RETLW	0			; RETURN

;******************************************************************************

UIR_WDT		CLRWDT				; CLEAR WDT
		BCF	STATUS,C		; CLEAR CARRY
		MOVLW	0x26			; LOAD 0x26
		SUBWF	UIR_BUFFER1,W		; SUBSTRACT
		BTFSC	STATUS,C		; TEST CARRY
		GOTO	MAIN			; RETURN HOME
UIR_WDT2	RLF	UIR_OUTPUT6,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT5,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT4,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT3,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT2,F		; SHIFT BUFFER
		RLF	UIR_OUTPUT1,F		; SHIFT BUFFER
		DECFSZ	UIR_BUFFER1,F		; DEC BUFFER
		GOTO	UIR_WDT2		; LOOP
		GOTO	RECEIVED_CMD		; PROCESS COMMAND

;******************************************************************************

READ_CONFIG	MOVLW	RAM_BUFFER1		; LOAD POINTER
		CALL	READ_EEPROM		; READ PAGE 1
		MOVLW	RAM_BUFFER4		; LOAD POINTER
		GOTO	READ_EEPROM		; READ PAGE 2

WRITE_CONFIG	MOVLW	RAM_BUFFER1		; LOAD POINTER
		CALL	WRITE_EEPROM		; WRITE PAGE 1
		MOVLW	RAM_BUFFER4		; LOAD POINTER
		GOTO	WRITE_EEPROM		; WRITE PAGE 2

;******************************************************************************

READ_EEPROM	MOVWF	EEPROM_ADDRESS		; STORE ADDRESS
		BSF	STATUS,IRP		; BANK 2 & 3
		CLRF	EEPROM_COUNTER		; CLEAR EEPROM COUNTER
READ_EEPROM_1	CLRWDT				; CLEAR WATCHDOG
		BSF	STATUS,RP1		; BANK 2
		BCF	STATUS,RP0		; BANK 2
		MOVF	EEPROM_COUNTER,W	; LOAD ADDRESS
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		IORLW	0x80			; YES, SET BIT
		MOVWF	EEADR			; SET ADDRESS
		BSF	STATUS,RP0		; BANK 3
		BCF	EECON1,EEPGD		; DATA MEMORY
		BSF	EECON1,RD		; START READ OPERATION
		BCF	STATUS,RP0		; BANK 2
		MOVF	EEPROM_COUNTER,W	; LOAD COUNTER
		XORLW	0x60			; COMPARE 96
		BTFSC	STATUS,Z		; 0 ?
		GOTO	READ_CONFIG_0		; YES
		MOVF	EEPROM_COUNTER,W	; LOAD COUNTER
		XORLW	0x61			; COMPARE 97
		BTFSC	STATUS,Z		; 0 ?
		GOTO	READ_CONFIG_1		; YES
		MOVF	EEPROM_ADDRESS,W	; LOAD ADDRESS
		ADDWF	EEPROM_COUNTER,W	; ADD OFFSET
		MOVWF	FSR			; SET POINTER
READ_EEPROM_2	COMF	EEDATA,W		; LOAD DATA
		MOVWF	INDF			; STORE DATA
		INCF	EEPROM_COUNTER,F	; INC COUNTER
		MOVLW	0x62			; LOAD 98
		XORWF	EEPROM_COUNTER,W	; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	READ_EEPROM_1		; NO, NEXT BYTE
		BCF	STATUS,RP1		; BANK 0
		BCF	STATUS,RP0		; BANK 0
		BCF	STATUS,IRP		; BANK 0 & 1
		COMF	TOGGLE_TEMP1,W		; LOAD & INVERT TEMP 1
		MOVWF	TOGGLE1			; STORE TOGGLE 1
		COMF	TOGGLE_TEMP2,W		; LOAD & INVERT TEMP 2
		MOVWF	TOGGLE2			; STORE TOGGLE 2
		RETURN				; RETURN

READ_CONFIG_0	MOVLW	TOGGLE_TEMP1		; LOAD CONFIG BYTE
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		ADDLW	0x01			; YES, ADD 1
		MOVWF	FSR			; SET POINTER
		GOTO	READ_EEPROM_2		; READ DATA
READ_CONFIG_1	MOVLW	MASTER1			; LOAD CONFIG BYTE
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		ADDLW	0x01			; YES, ADD 1
		MOVWF	FSR			; SET POINTER
		GOTO	READ_EEPROM_2		; READ DATA

;******************************************************************************

WRITE_EEPROM	MOVWF	EEPROM_ADDRESS		; STORE ADDRESS
		COMF	TOGGLE1,W		; LOAD & INVERT TOGGLE 1
		MOVWF	TOGGLE_TEMP1		; STORE IN TEMP 1
		COMF	TOGGLE2,W		; LOAD & INVERT TOGGLE 2
		MOVWF	TOGGLE_TEMP2		; STORE IN TEMP 2
		BSF	STATUS,IRP		; BANK 2 & 3
		CLRF	EEPROM_COUNTER		; CLEAR EEPROM COUNTER
WRITE_EEPROM_1	CLRWDT				; CLEAR WATCHDOG
		BSF	STATUS,RP1		; BANK 3
		BSF	STATUS,RP0		; BANK 3
		BTFSC	EECON1,WR		; WAIT FOR WRITE TO FINISH
		GOTO	WRITE_EEPROM_1		; LOOP
		BCF	STATUS,RP0		; BANK 2
		MOVF	EEPROM_COUNTER,W	; LOAD ADDRESS
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		IORLW	0x80			; YES, SET BIT
		MOVWF	EEADR			; SET ADDRESS
		MOVF	EEPROM_COUNTER,W	; LOAD COUNTER
		XORLW	0x60			; COMPARE 96
		BTFSC	STATUS,Z		; 0 ?
		GOTO	WRITE_CONFIG_0		; YES
		MOVF	EEPROM_COUNTER,W	; LOAD COUNTER
		XORLW	0x61			; COMPARE 97
		BTFSC	STATUS,Z		; 0 ?
		GOTO	WRITE_CONFIG_1		; YES
		MOVF	EEPROM_ADDRESS,W	; LOAD ADDRESS
		ADDWF	EEPROM_COUNTER,W	; ADD OFFSET
		MOVWF	FSR			; SET POINTER
WRITE_EEPROM_2	COMF	INDF,W			; LOAD DATA
		MOVWF	EEPROM_TEMP		; STORE IN TEMP
		MOVWF	EEDATA			; DATA TO WRITE
		BSF	STATUS,RP0		; BANK 3
		BCF	EECON1,EEPGD		; DATA MEMORY
		BSF	EECON1,WREN		; ENABLE WRITES
		BCF	INTCON,GIE		; DISABLE IRQ
		MOVLW	0x55			; LOAD 0x55
		MOVWF	EECON2			; TO EECON2
		MOVLW	0xAA			; LOAD 0xAA
		MOVWF	EECON2			; TO EECON2
		BSF	EECON1,WR		; START WRITE OPERATION
		BSF	INTCON,GIE		; ENABLE IRQ
		BCF	EECON1,WREN		; DISABLE WRITES
WRITE_EEPROM_3	CLRWDT				; CLEAR WATCHDOG
		BTFSC	EECON1,WR		; WAIT FOR WRITE TO FINISH
		GOTO	WRITE_EEPROM_3		; LOOP
		BCF	STATUS,RP0		; BANK 2
		MOVF	EEPROM_COUNTER,W	; LOAD ADDRESS
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		IORLW	0x80			; YES, SET BIT
		MOVWF	EEADR			; SET ADDRESS
		BSF	STATUS,RP0		; BANK 3
		BCF	EECON1,EEPGD		; DATA MEMORY
		BSF	EECON1,RD		; START READ OPERATION
		BCF	STATUS,RP0		; BANK 2
		MOVF	EEDATA,W		; LOAD DATA
		XORWF	EEPROM_TEMP,W		; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	WRITE_EEPROM_1		; WRITE ERROR, TRY AGAIN
		INCF	EEPROM_COUNTER,F	; INC COUNTER
		MOVLW	0x62			; LOAD 98
		XORWF	EEPROM_COUNTER,W	; COMPARE
		BTFSS	STATUS,Z		; 0 ?
		GOTO	WRITE_EEPROM_1		; NO, NEXT BYTE
		BCF	STATUS,RP1		; BANK 0
		BCF	STATUS,RP0		; BANK 0
		BCF	STATUS,IRP		; BANK 0 & 1
		RETURN				; RETURN

WRITE_CONFIG_0	MOVLW	TOGGLE_TEMP1		; LOAD CONFIG BYTE
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		ADDLW	0x01			; YES, ADD 1
		MOVWF	FSR			; SET POINTER
		GOTO	WRITE_EEPROM_2		; WRITE DATA
WRITE_CONFIG_1	MOVLW	MASTER1			; LOAD CONFIG BYTE
		BTFSC	EEPROM_ADDRESS,7	; PAGE 2?
		ADDLW	0x01			; YES, ADD 1
		MOVWF	FSR			; SET POINTER
		GOTO	WRITE_EEPROM_2		; WRITE DATA

;******************************************************************************

WAIT_BITHALF	NOP				; NOP
		MOVLW	0x0B			; 9600Bd @ 4MHz
		GOTO	WAIT_LOAD		; WAIT 418 US
		RETURN				; RETURN
WAIT_BITTIME	NOP				; NOP
		NOP				; NOP
		MOVLW	0x17			; 9600Bd @ 4MHz
WAIT_LOAD	MOVWF	SMALL_COUNT		; LOAD INTO SMALL_COUNT
WAIT_4U		CLRWDT				; CLEAR WDT
		DECFSZ	SMALL_COUNT,F		; DEC
		GOTO	WAIT_4U			; 4US LOOP
		RETURN				; RETURN
WAIT_MS		MOVWF	LARGE_COUNT		; LOAD MILISEC
WAIT_LOOP	CALL	WAIT_1MS		; 1MS DELAY
		DECFSZ	LARGE_COUNT,F		; DEC
		GOTO	WAIT_LOOP		; LOOP
		RETURN				; RETURN
WAIT_1MS	MOVLW	0xFA			; 250 X 4US = 1MS
		MOVWF	SMALL_COUNT		; LOAD INTO SMALL_COUNT
		GOTO	WAIT_4U			; 4US LOOP

;******************************************************************************

	END
